@@ -34,6 +34,14 @@ def screen_bell() -> None:
     pass
 
 
+def screen_normal_keypad_mode() -> None:
+    write('\x1b>')
+
+
+def screen_alternate_keypad_mode() -> None:
+    write('\x1b=')
+
+
 def screen_cursor_position(y: int, x: int) -> None:
     write(CSI + '%s;%sH' % (y, x))
 
@@ -42,10 +50,26 @@ def screen_cursor_forward(amt: int) -> None:
     write(CSI + '%sC' % amt)
 
 
+def screen_save_cursor() -> None:
+    write('\x1b7')
+
+
+def screen_restore_cursor() -> None:
+    write('\x1b8')
+
+
 def screen_cursor_back1(amt: int) -> None:
     write(CSI + '%sD' % amt)
 
 
+def screen_save_modes() -> None:
+    write(CSI + '?s')
+
+
+def screen_restore_modes() -> None:
+    write(CSI + '?r')
+
+
 def screen_designate_charset(which: int, to: int) -> None:
     w = '()'[int(which)]
     t = chr(int(to))
@@ -60,8 +84,8 @@ def screen_cursor_to_column(c: int) -> None:
     write(CSI + '%dG' % c)
 
 
-def screen_cursor_to_line(l: int) -> None:
-    write(CSI + '%dd' % l)
+def screen_cursor_to_line(ln: int) -> None:
+    write(CSI + '%dd' % ln)
 
 
 def screen_set_mode(x: int, private: bool) -> None:
@@ -80,6 +104,34 @@ def screen_indexn(n: int) -> None:
     write(CSI + '%dS' % n)
 
 
+def screen_delete_characters(count: int) -> None:
+    write(CSI + '%dP' % count)
+
+
+def screen_push_colors(which: int) -> None:
+    write(CSI + '%d#P' % which)
+
+
+def screen_pop_colors(which: int) -> None:
+    write(CSI + '%d#Q' % which)
+
+
+def screen_report_colors() -> None:
+    write(CSI + '#R')
+
+
+def screen_repeat_character(num: int) -> None:
+    write(CSI + '%db' % num)
+
+
+def screen_insert_characters(count: int) -> None:
+    write(CSI + '%d@' % count)
+
+
+def screen_scroll(count: int) -> None:
+    write(CSI + '%dS' % count)
+
+
 def screen_erase_in_display(how: int, private: bool) -> None:
     write(CSI + ('?' if private else '') + str(how) + 'J')
 
@@ -108,6 +160,10 @@ def screen_linefeed() -> None:
     write('\n')
 
 
+def screen_tab() -> None:
+    write('\t')
+
+
 def screen_backspace() -> None:
     write('\x08')
 
@@ -137,20 +193,29 @@ def report_device_attributes(mode: int, char: int) -> None:
     write(CSI + x + 'c')
 
 
+def screen_decsace(mode: int) -> None:
+    write(CSI + str(mode) + '*x')
+
+
+def screen_set_8bit_controls(mode: int) -> None:
+    write('\x1b ' + ('G' if mode else 'F'))
+
+
 def write_osc(code: int, string: str = '') -> None:
     if string:
         string = ';' + string
     write(OSC + str(code) + string + '\x07')
 
 
-set_dynamic_color = set_color_table_color = write_osc
+set_dynamic_color = set_color_table_color = process_cwd_notification = write_osc
 
 
 def replay(raw: str) -> None:
+    specials = {'draw', 'set_title', 'set_icon', 'set_dynamic_color', 'set_color_table_color', 'process_cwd_notification'}
     for line in raw.splitlines():
         if line.strip() and not line.startswith('#'):
             cmd, rest = line.partition(' ')[::2]
-            if cmd in {'draw', 'set_title', 'set_icon', 'set_dynamic_color', 'set_color_table_color'}:
+            if cmd in specials:
                 globals()[cmd](rest)
             else:
                 r = map(int, rest.split()) if rest else ()
